
import javax.vecmath.*;

public class Camera
{
  Vector3f posVec, viewVec, rightVec, upVec;
  float rotCamX, rotCamY, rotCamZ;
  float rotateSpeed = 10f;
  float translateSpeed = .3f;
  ArrayList animations = new ArrayList();

  public Camera()
  {
    resetCamera();
  }

  public void resetCamera()
  {
   // posVec = new Vector3f(0f, 25f, -45f);
    posVec = new Vector3f(0f, 10f, 70f);
   
    viewVec = new Vector3f(0f, 0f, -1f);
    rightVec = new Vector3f(1f, 0f, 0f);
    upVec = new Vector3f(0f, 1f, 0f);  
  }

  void animate(GL gl, GLU glu)
  {
    for (int i = animations.size() - 1; i >= 0; i--)
    {
      Animation a = (Animation) animations.get(i);
      if (a.update(gl, glu, millis()))
      {
        animations.remove(a);
      }
    }
  }

  void renderCam(GL gl, GLU glu)
  {
    //viewpoint is the point at which the camera looks:
    Vector3f viewpoint = new Vector3f(posVec);
    viewpoint.add(viewVec);

    glu.gluLookAt
      (
    posVec.x, posVec.y, posVec.z,
    viewpoint.x, viewpoint.y, viewpoint.z,
    upVec.x, upVec.y, upVec.z
      );
  }


  void moveCam(Vector3f directionVec)
  {
    posVec.add(directionVec);
  }

  void rotateCamX (float angle)
  {
    rotCamX += angle;

    //Rotate viewdir around the right vector:
    Vector3f tmp1 = new Vector3f(viewVec); 
    tmp1.scale((float)Math.cos(Math.toRadians(angle)));
    Vector3f tmp2 = new Vector3f(upVec);
    tmp2.scale((float)Math.sin(Math.toRadians(angle)));
    tmp1.add(tmp2);

    viewVec.normalize(tmp1);	

    //now compute the new UpVector (by cross product)
    upVec.cross(viewVec, rightVec);
    upVec.scale(-1f);
  }

  void rotateCamY (float angle)
  {
    rotCamY += angle;

    //Rotate viewdir around the up vector:
    Vector3f tmp1 = new Vector3f(viewVec); 
    tmp1.scale((float)Math.cos(Math.toRadians(angle)));
    Vector3f tmp2 = new Vector3f(rightVec);
    tmp2.scale((float)Math.sin(Math.toRadians(angle)));
    tmp1.sub(tmp2);

    viewVec.normalize(tmp1);	

    //now compute the new UpVector (by cross product)
    rightVec.cross(viewVec, upVec);
  }

  void rotateCamZ (float angle)
  {
    rotCamZ += angle;

    //Rotate viewdir around the right vector:
    Vector3f tmp1 = new Vector3f(rightVec); 
    tmp1.scale((float)Math.cos(Math.toRadians(angle)));
    Vector3f tmp2 = new Vector3f(upVec);
    tmp2.scale((float)Math.sin(Math.toRadians(angle)));
    tmp1.add(tmp2);

    rightVec.normalize(tmp1);	

    //now compute the new UpVector (by cross product)
    upVec.cross(viewVec, rightVec);
    upVec.scale(-1f);
  }

  void moveCamX( float dist )
  {
    Vector3f tmp1 = new Vector3f(rightVec);
    tmp1.scale(dist);
    posVec.add(tmp1);
  }

  void moveCamY( float dist )
  {
    Vector3f tmp1 = new Vector3f(upVec);
    tmp1.scale(dist);
    posVec.add(tmp1);
  }

  void moveCamZ( float dist )
  {
    Vector3f tmp1 = new Vector3f(viewVec);
    tmp1.scale(-dist);
    posVec.add(tmp1);

  }

}









