
import processing.opengl.*;
import javax.media.opengl.*; 
import javax.media.opengl.glu.*;
import com.sun.opengl.util.texture.*; 
import javax.vecmath.*;
import com.sun.opengl.util.BufferUtil;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;

float FOVY = 45f;
float NEAR = .1f;
float FAR = 1000f;

PGraphicsOpenGL pgl;
GL gl;
GLU glu;
Camera cam;

int[] viewportBounds = new int[4];
int SIZE = 64;
float[][][] mesh = new float[SIZE][SIZE][3];
float wave_movement = 0.0f;
float waveInc = 0.00003f;
void setup() 
{
  size(860, 500, OPENGL); 

  frame.setResizable(true); 
  viewportBounds = new int[]
  { 
    0,0, width, height               
  };

  cam = new Camera();

  background(0);
}

void initProjection(GL gl, GLU glu)
{
  gl.glViewport(0, 0, width, height);  
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(FOVY, (float)width/(float)height, NEAR, FAR);
}

void initModelview(GL gl)
{
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
}

int programID = -1;
int waveVarID = -1;
boolean firstTime = true;

void draw()
{
  background(0); //clear screen

  //get OpenGL context
  pgl = (PGraphicsOpenGL) g; 
  gl = pgl.gl;
  glu = pgl.glu;

  //override Processing defaults to work with normal OpenGL matrices
  initProjection(gl, glu);
  initModelview(gl);

  //handle camera view
  cam.renderCam(gl, glu);

  if (firstTime == true)
  {
    //set to wire mode
    gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);

    //load in shader program
    programID = initShaders("vertexDis.vert", "vertexDis.frag");
    waveVarID = gl.glGetAttribLocation(programID, "wave");

    for (int x = 0; x < SIZE; x++) 
    {
      for (int z = 0; z < SIZE; z++) 
      {
        mesh[x][z][0] = (float) (SIZE / 2) - x;     
        mesh[x][z][1] = 0.0f;                       
        mesh[x][z][2] = (float) (SIZE / 2) - z;   
      }
    }

    firstTime = false;
  }

  if (programID <= 0)
  {
    println("ERROR - couldn't load shader program!");
  }
  else
  { 
    //Bind GLSL Program to overwrite fixed functionality
    gl.glUseProgram(programID);

    // Start Drawing Our Mesh
    for (int x = 0; x < SIZE - 1; x++) 
    {
      // Draw A Triangle Strip For Each Column Of Our Mesh
      gl.glBegin(GL.GL_TRIANGLE_STRIP);

      for (int z = 0; z < SIZE - 1; z++) 
      {
        // Set The Wave Parameter Of Our Shader To The Incremented Wave Value From Our Main Program
        gl.glVertexAttrib1f(waveVarID, wave_movement);
        gl.glColor3f(0.5f, 1f, 0.5f);

        gl.glVertex3f(mesh[x][z][0], mesh[x][z][1], mesh[x][z][2]);        // Draw Vertex
        gl.glVertex3f(mesh[x + 1][z][0], mesh[x + 1][z][1], mesh[x + 1][z][2]);    // Draw Vertex
        wave_movement += waveInc;                                     // Increment Our Wave Movement
        if (wave_movement > Math.PI * 2f) 
        {                                // Prevent Crashing
          wave_movement = 0.0f;
        }
      }
      gl.glEnd();
    }



    //Un-bind GLSL Program to return to fixed functionality
    gl.glUseProgram(0);
  }
}

void resetProcessingMatrices(GL gl, GLU glu)
{
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(60, width/(float)height, .1f, 1000f);
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
  glu.gluLookAt(width/2.0f, height/2.0f, (height/2.0f) / tan(PI*60.0f / 360.0f), width/2.0f, height/2.0f, 0, 0, 1, 0); 
}



void keyPressed()
{
  switch(key)
  {
  case 'a':
    cam.moveCamX(-cam.translateSpeed);
    break;
  case 'z':
    cam.moveCamX(cam.translateSpeed);
    break;
  case 's':
    cam.moveCamY(cam.translateSpeed);
    break;
  case 'x':
    cam.moveCamY(-cam.translateSpeed);
    break;
  case 'd':
    cam.moveCamZ(-cam.translateSpeed);
    break;
  case 'c':
    cam.moveCamZ(cam.translateSpeed);
    break;

  case 'g':
    cam.rotateCamY(cam.rotateSpeed);
    break;
  case 'b':
    cam.rotateCamY(-cam.rotateSpeed);
    break;
  case 'h':
    cam.rotateCamZ(-cam.rotateSpeed);
    break;
  case 'n':
    cam.rotateCamZ(cam.rotateSpeed);
    break;
  case 'j':
    cam.rotateCamX(-cam.rotateSpeed);
    break;
  case 'm':
    cam.rotateCamX(cam.rotateSpeed);
    break;

  case 'q':
    cam.resetCamera();
    break;

  case '1':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 0f, 180f, 0f, 0f, 0f, 0f));
    break;
  case '2':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 0f, 0f, 180f, 0f, 0f, 0f));
    break;
  case '3':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 180f, 0f, 0f, 0f, 0f, 0f));
    break;
  case '4':  
    cam.animations.add(new Animation(cam, millis(), millis() + 300, 0f, 0f, 0f, 0f, 0f, +5f));
    break;
  case '5':  
    cam.animations.add(new Animation(cam, millis(), millis() + 300, 0f, 0f, 0f, 0f, 0f, -5f));
    break;
  }
}

void mouseDragged() 
{
  float rate = .5f;
  cam.rotateCamY((mouseX-pmouseX) * rate);
}




















